// Generated by CoffeeScript 1.7.1
var parse, _,
  __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

_ = require('lodash');

parse = (function() {
  var _mapCompareOp;

  function parse() {
    this.parseWhere = __bind(this.parseWhere, this);
    this._parseObject = __bind(this._parseObject, this);
    this._parseTextSearchTarget = __bind(this._parseTextSearchTarget, this);
    this._parseTarget = __bind(this._parseTarget, this);
    this._getTheOnlyKey = __bind(this._getTheOnlyKey, this);
    this.parseOffset = __bind(this.parseOffset, this);
    this.parseLimit = __bind(this.parseLimit, this);
    this.parseOrderBy = __bind(this.parseOrderBy, this);
    this.parseSelect = __bind(this.parseSelect, this);
    this.parseOneSelectField = __bind(this.parseOneSelectField, this);
    this.parseTSIndex = __bind(this.parseTSIndex, this);
    this.parseIndexField = __bind(this.parseIndexField, this);
    this.isSupportLanguage = __bind(this.isSupportLanguage, this);
    this.setJSONColumnName = __bind(this.setJSONColumnName, this);
    this.jsonColumnName = "data";
  }

  parse.prototype.setJSONColumnName = function(jsonColumnName) {
    return this.jsonColumnName = jsonColumnName;
  };

  parse.prototype.isSupportLanguage = function(lang) {
    return /^(danish|dutch|english|finnish|french|german|hungarian|italian|norwegian|portuguese|romanian|russian|spanish|swedish|turkish|simple)$/.test(lang);
  };

  parse.prototype.parseIndexField = function(field, isText) {
    var f, fieldArray, fieldString, i, _i, _ref;
    if (isText == null) {
      isText = false;
    }
    fieldString = this.jsonColumnName;
    fieldArray = field.trim().split(":");
    if (fieldArray.length === 0) {
      throw "invalid key name:" + field + ".";
    }
    for (i = _i = 0, _ref = fieldArray.length - 1; 0 <= _ref ? _i <= _ref : _i >= _ref; i = 0 <= _ref ? ++_i : --_i) {
      f = fieldArray[i].trim();
      if (/[\s]+/.test(f) || /^\$[0-9]*$/.test(f)) {
        throw "invalid key name:" + field + ".";
      }
      if (i === (fieldArray.length - 1) && isText) {
        fieldString += "->>\'" + f + "\'";
      } else {
        fieldString += "->\'" + f + "\'";
      }
    }
    return fieldString;
  };

  parse.prototype.parseTSIndex = function(field, language) {
    if (language == null) {
      language = "simple";
    }
    field = field.trim();
    if (field.split(":").length > 1) {
      throw "only support outermost fields";
    }
    if (typeof field !== 'string' || /[\s]+/.test(field) || /^\$[0-9]*$/.test(field)) {
      throw "invalid key name:" + field + ".";
    }
    if (!this.isSupportLanguage(language)) {
      throw "unsupport language";
    }
    return this.jsonColumnName + ("->>\'" + field + "\'");
  };

  parse.prototype.parseOneSelectField = function(field, need$AsField) {
    var fieldArray, fieldString, found$As, k;
    if (need$AsField == null) {
      need$AsField = false;
    }
    fieldString = this.jsonColumnName;
    found$As = false;
    fieldArray = field.split(":");
    if (fieldArray.length === 1) {
      k = fieldArray[0].trim();
      if (/[\s]+/.test(k) || /^[0-9]+/.test(k) || k.length === 0) {
        throw "invalid key nam: " + fieldArray;
      }
      fieldString += "->\'" + k + "\'";
      if (need$AsField) {
        fieldString = "to_json(" + fieldString + ")";
        fieldString += " as " + k;
      }
      return fieldString;
    }
    _.forEach(fieldArray, (function(_this) {
      return function(k) {
        k = k.trim();
        if (/[\s]+/.test(k)) {
          throw "invalid key nam: " + k;
        } else if (/^\$[0-9]*$/.test(k)) {
          return fieldString += "->" + (Number(k.substr(1)));
        } else if (/^(\$as|\$AS)$/.test(k)) {
          if (!need$AsField) {
            throw "unexpected $as found at " + field;
          }
          fieldString = "to_json(" + fieldString + ")";
          fieldString += " as ";
          return found$As = true;
        } else if (!/^[0-9]+/.test(k)) {
          if (k.length > 0) {
            if (found$As) {
              return fieldString += k;
            } else {
              return fieldString += "->\'" + k + "\'";
            }
          }
        } else {
          throw "invalid key naming: " + k;
        }
      };
    })(this));
    if (!found$As && need$AsField) {
      if (fieldArray.length === 1) {
        fieldString = "to_json(" + fieldArray[0] + ")";
        fieldString += " as " + fieldArray[0];
        found$As = true;
      }
      if (!found$As) {
        throw "$as field is needed";
      }
    }
    if (fieldString === this.jsonColumnName) {
      return null;
    } else {
      return fieldString;
    }
  };

  parse.prototype.parseSelect = function(select, required) {
    var selectString;
    if (required == null) {
      required = false;
    }
    if (select == null) {
      if (required) {
        throw "need select condition for query";
      } else {
        return "*";
      }
    }
    if (/[\*]{1,}/.test(select)) {
      return "*";
    }
    selectString = "";
    _.forEach(select.split(" "), (function(_this) {
      return function(field) {
        var fieldString;
        fieldString = _this.parseOneSelectField(field, true);
        if (!fieldString) {
          throw "invalid select condition string";
        }
        if (selectString === "") {
          return selectString += fieldString;
        } else {
          return selectString += "," + fieldString;
        }
      };
    })(this));
    return selectString;
  };

  parse.prototype.parseOrderBy = function(orderBy, required) {
    var ascend, orderByInArray, orderByString;
    if (required == null) {
      required = false;
    }
    if (orderBy == null) {
      if (required) {
        throw "need orderBy condition for query";
      } else {
        return null;
      }
    }
    if (/[\*]{1,}/.test(orderBy)) {
      return "orderBy field can't accept *";
    }
    ascend = "ASC";
    orderByInArray = orderBy.trim().split(" ");
    if (orderByInArray.length === 0) {
      throw "invalid order by clause";
    }
    if (orderByInArray.length > 1 && /^(ASC|DESC)$/.test(orderByInArray[1].toUpperCase())) {
      ascend = orderByInArray[1];
    }
    orderByString = this.parseOneSelectField(orderByInArray[0]);
    if (orderByString != null) {
      return orderByString + " " + ascend;
    } else {
      return null;
    }
  };

  parse.prototype.parseLimit = function(limit, required) {
    var number;
    if (required == null) {
      required = false;
    }
    if (limit == null) {
      if (required) {
        throw "need limit condition for query";
      } else {
        return null;
      }
    }
    if (_.isNumber(limit)) {
      return limit;
    }
    number = Number(limit);
    if (number === NaN) {
      throw "invlaid limit number";
    }
    return number;
  };

  parse.prototype.parseOffset = function(offset, required) {
    var number;
    if (required == null) {
      required = false;
    }
    if (offset == null) {
      if (required) {
        throw "need offset condition for query";
      } else {
        return null;
      }
    }
    if (_.isNumber(offset)) {
      return offset;
    }
    number = Number(offset);
    if (number === NaN) {
      throw "invlaid offset number";
    }
    return number;
  };

  _mapCompareOp = {
    eq: "=",
    ne: "<>",
    gt: ">",
    ge: ">=",
    lt: "<",
    le: "<=",
    like: "~~",
    ilike: "~~*",
    text: "@@",
    all: "?&",
    any: "?|",
    inc: "@>"
  };

  parse.prototype._getTheOnlyKey = function(o) {
    var k, theKey;
    if (!_.isPlainObject(o)) {
      return null;
    }
    if (_.size(o) !== 1) {
      throw "condition can just have one key";
    }
    for (k in o) {
      if (o.hasOwnProperty(k) && typeof k !== 'function') {
        theKey = k;
      }
    }
    return theKey;
  };

  parse.prototype._parseTarget = function(value, op) {
    var arrString, asText, target;
    target = null;
    asText = /^(~~)(\*){0,1}$/.test(op);
    switch (typeof value) {
      case 'object':
        if (_.isArray(value)) {
          if (/^(\?\&|\?\|)$/.test(op)) {
            arrString = "";
            _.forEach(value, function(e) {
              if (arrString === "") {
                return arrString += "\'" + e + "\'";
              } else {
                return arrString += ",\'" + e + "\'";
              }
            });
            target = "array[" + arrString + "]";
          } else {
            target = "\'" + (JSON.stringify(value)) + "\'";
          }
        } else {
          if (/^(@>)$/.test(op)) {
            throw "$inc can check if array includes string/number/boolean, but not object";
          }
          target = "\'" + (JSON.stringify(value)) + "\'";
        }
        break;
      case 'string':
        target = "\'" + value + "\'";
        asText = true;
        break;
      case 'boolean':
        target = "\'" + value + "\'";
        break;
      case 'number':
        target = "\'" + value + "\'";
    }
    return {
      target: target,
      asText: asText
    };
  };

  parse.prototype._parseTextSearchTarget = function(value) {
    var key, language, target;
    target = null;
    language = "simple";
    if (_.isPlainObject(value)) {
      key = this._getTheOnlyKey(value).toLowerCase();
      if (!isSupportLanguage(key)) {
        throw "the search language " + key + " is not support. Only suports danish, dutch, english, finnish,french,german,hungarian,italian,norwegian,portuguese,romanian,russian,spanish,swedish and turkish.";
      }
      language = key;
      value = value[key];
    }
    if (typeof value !== 'string') {
      throw "can not search non-string element";
    }
    target = "\'" + value + "\'";
    return {
      target: target,
      language: language
    };
  };

  parse.prototype._parseObject = function(object) {
    var asText, compareOp, currKey, currObject, expect, i, keyArray, language, target, tmp, _i, _ref, _ref1, _ref2, _ref3;
    if (!_.isPlainObject(object)) {
      throw "condition is not a object";
    }
    currObject = object;
    compareOp = null;
    keyArray = [];
    while (true) {
      currKey = this._getTheOnlyKey(currObject);
      if (currKey === null && compareOp === null) {
        compareOp = _mapCompareOp["eq"];
        _ref = this._parseTarget(currObject, "="), target = _ref.target, asText = _ref.asText;
        break;
      }
      if (/^(\$eq|\$ne|\$gt|\$ge|\$lt|\$le|\$all|\$any|\$inc|\$text|\$like|\$ilike)$/.test(currKey.toLowerCase())) {
        compareOp = _mapCompareOp[currKey.substr(1).toLowerCase()];
        if (compareOp === "@@") {
          _ref1 = this._parseTextSearchTarget(currObject[currKey]), language = _ref1.language, target = _ref1.target;
        } else {
          _ref2 = this._parseTarget(currObject[currKey], compareOp), target = _ref2.target, asText = _ref2.asText;
        }
        break;
      } else {
        if (/^\$[0-9]*$/.test(currKey)) {
          tmp = Number(currKey.substr(1));
          keyArray.push("" + tmp);
        } else {
          keyArray.push("\'" + currKey + "\'");
        }
        currObject = currObject[currKey];
      }
    }
    expect = this.jsonColumnName;
    if (compareOp === "@@") {
      if (keyArray.length === 0) {
        throw "can not search the entire document, must specify a field";
      }
      if (keyArray.length !== 1) {
        throw "invalid text search format";
      }
      expect = "->>" + keyArray[0];
      return "(" + ("to_tsvector(\'" + language + "\', " + expect + ")") + " " + compareOp + " " + ("to_tsquery(" + target + ")") + ")";
    } else {
      if (keyArray.length > 1) {
        for (i = _i = 0, _ref3 = keyArray.length - 2; 0 <= _ref3 ? _i <= _ref3 : _i >= _ref3; i = 0 <= _ref3 ? ++_i : --_i) {
          expect += "->" + keyArray[i];
        }
      }
      if (keyArray.length >= 1) {
        if (asText) {
          expect += "->>" + keyArray[keyArray.length - 1];
        } else {
          expect += "->" + keyArray[keyArray.length - 1];
        }
      }
      return "(" + expect + " " + compareOp + " " + target + ")";
    }
  };

  parse.prototype.parseWhere = function(query, required) {
    var opString, queryString, theKey;
    if (required == null) {
      required = false;
    }
    if (query == null) {
      if (required) {
        throw "need where condition for query";
      } else {
        return null;
      }
    }
    theKey = this._getTheOnlyKey(query);
    if (!/^(\$or|\$OR|\$not|\$NOT|\$and|\$AND)$/.test(theKey)) {
      return this._parseObject(query);
    } else {
      opString = theKey.substr(1).toLowerCase();
      if (opString === 'not') {
        if (!_.isObject(query[theKey])) {
          throw "$not takes object as parameter";
        }
        queryString = this.parseWhere(query[theKey]);
        return "(not " + queryString + ")";
      } else {
        if (!_.isArray(query[theKey])) {
          query[theKey] = [query[theKey]];
        }
        if (query[theKey].length === 0) {
          throw "$and/$or takes at least one condition";
        }
        queryString = "";
        _.forEach(query[theKey], (function(_this) {
          return function(q) {
            var childQueryString;
            childQueryString = _this.parseWhere(q);
            if (queryString === "") {
              return queryString += childQueryString;
            } else {
              return queryString += " " + opString + " " + childQueryString;
            }
          };
        })(this));
        return "(" + queryString + ")";
      }
    }
  };

  return parse;

})();

module.exports = new parse();
