// Generated by CoffeeScript 1.7.1
(function() {
  var expect, expectFail, expectPass, parser, _;

  parser = require('../lib/parse');

  expect = require('chai').expect;

  _ = require('lodash');

  expectPass = function(expFn, result) {
    var error, r;
    try {
      r = expFn();
      return expect(r).to.equal(result);
    } catch (_error) {
      error = _error;
      return expect(error).to.equal(void 0);
    }
  };

  expectFail = function(expFn) {
    var error;
    try {
      expFn();
      return expect(error).to.equal(void 0);
    } catch (_error) {
      error = _error;
      return expect(error).not.to.equal(null);
    }
  };

  describe('pg json parser test', function() {
    it('_parseObject fail on bad arguments', function(done) {
      expectFail(function() {
        return parser._parseObject({
          a: {
            b: 1
          }
        });
      });
      expectFail(function() {
        return parser._parseObject({
          a: {
            $eq: 1
          },
          b: {
            $eq: 2
          }
        });
      });
      return done();
    });
    it('_parseObject pass on good arguments', function(done) {
      expectPass(function() {
        return parser._parseObject({
          a: {
            $eq: 1
          }
        });
      }, "(data->\'a\' = '1')");
      expectPass(function() {
        return parser._parseObject({
          a: {
            $eq: false
          }
        });
      }, "(data->\'a\' = 'false')");
      expectPass(function() {
        return parser._parseObject({
          a: {
            $eq: {
              c: "hello",
              d: 120,
              e: true
            }
          }
        });
      }, "(data->\'a\' = \'{\"c\":\"hello\",\"d\":120,\"e\":true}\')");
      expectPass(function() {
        return parser._parseObject({
          a: {
            $eq: [120, "hello", true]
          }
        });
      }, "(data->\'a\' = \'[120,\"hello\",true]\')");
      expectPass(function() {
        var query;
        return query = parser._parseObject({
          a: {
            $0: {
              $eq: "apple"
            }
          }
        });
      }, "(data->\'a\'->>0 = \'apple\')");
      expectPass(function() {
        var query;
        return query = parser._parseObject({
          a: {
            $0: {
              b: {
                $eq: "apple"
              }
            }
          }
        });
      }, "(data->\'a\'->0->>\'b\' = \'apple\')");
      expectPass(function() {
        return parser._parseObject({
          $all: ["sku", "price"]
        });
      }, "(data ?& array[\'sku\',\'price\'])");
      expectPass(function() {
        return parser._parseObject({
          $any: ["sku", "price"]
        });
      }, "(data ?| array[\'sku\',\'price\'])");
      return done();
    });
    it("parseWhere fails on bad arguments", function(done) {
      expectFail(function() {
        return parser.parseWhere("notObject");
      });
      expectFail(function() {
        return parser.parseWhere({
          a: 1,
          b: 2
        });
      });
      expectFail(function() {
        return parser.parseWhere({
          $not: [
            {
              a: 1
            }, {
              b: 2
            }
          ]
        });
      });
      return done();
    });
    it("$or/$and should pass if followed by a object or array with one elment", function(done) {
      expectPass(function() {
        return parser.parseWhere({
          $and: {
            a: {
              $eq: 1
            }
          }
        });
      }, "((data->\'a\' = \'1\'))");
      expectPass(function() {
        return parser.parseWhere({
          $or: {
            a: {
              $eq: 1
            }
          }
        });
      }, "((data->\'a\' = \'1\'))");
      expectPass(function() {
        return parser.parseWhere({
          $and: [
            {
              a: {
                $eq: 1
              }
            }
          ]
        });
      }, "((data->\'a\' = \'1\'))");
      expectPass(function() {
        return parser.parseWhere({
          $or: [
            {
              a: {
                $eq: 1
              }
            }
          ]
        });
      }, "((data->\'a\' = \'1\'))");
      return done();
    });
    it("queryWhere should pass good arguments", function(done) {
      expectPass(function() {
        return parser.parseWhere({
          $or: [
            {
              a: {
                $gt: 20
              }
            }, {
              b: {
                $le: 10
              }
            }
          ]
        });
      }, "((data->\'a\' > \'20\') or (data->\'b\' <= \'10\'))");
      expectPass(function() {
        return parser.parseWhere({
          $and: [
            {
              a: {
                $gt: 20
              }
            }, {
              b: {
                $le: 10
              }
            }
          ]
        });
      }, "((data->\'a\' > \'20\') and (data->\'b\' <= \'10\'))");
      expectPass(function() {
        return parser.parseWhere({
          $and: [
            {
              $not: {
                a: {
                  $gt: 20
                }
              }
            }, {
              b: {
                $le: 10
              }
            }
          ]
        });
      }, "((not (data->\'a\' > \'20\')) and (data->\'b\' <= \'10\'))");
      expectPass(function() {
        return parser.parseWhere({
          $not: {
            $and: [
              {
                a: {
                  $gt: 20
                }
              }, {
                b: {
                  $eq: true
                }
              }, {
                c: {
                  $0: {
                    $lt: 10
                  }
                }
              }, {
                d: {
                  e: {
                    $eq: "apple"
                  }
                }
              }
            ]
          }
        });
      }, "(not ((data->\'a\' > \'20\') and (data->\'b\' = \'true\') and (data->\'c\'->0 < \'10\') and (data->\'d\'->>\'e\' = \'apple\')))");
      return done();
    });
    it('parseOneSelectField should pass good arguments', function(done) {
      expectPass(function() {
        return parser.parseOneSelectField("holiday");
      }, "data->\'holiday\'");
      expectPass(function() {
        return parser.parseOneSelectField("holiday", true);
      }, "to_json(data->\'holiday\') as holiday");
      expectPass(function() {
        return parser.parseOneSelectField("holiday:$0");
      }, "data->\'holiday\'->0");
      return done();
    });
    it('parseOneSelectField should fail on bad arguments', function(done) {
      expectPass(function() {
        return parser.parseOneSelectField(":");
      }, null);
      expectFail(function() {
        return parser.parseOneSelectField(" ");
      });
      expectFail(function() {
        return parser.parseOneSelectField("12");
      });
      expectFail(function() {
        return parser.parseOneSelectField("12afe");
      });
      expectFail(function() {
        return parser.parseOneSelectField("holiday:12afe");
      });
      expectFail(function() {
        return parser.parseOneSelectField("holiday:$0", true);
      });
      return done();
    });
    it('parseOneSelectField should pass on good arguments"', function(done) {
      expectPass(function() {
        return parser.parseOneSelectField("holiday:newYear:$as:newYearHoliday", true);
      }, "to_json(data->\'holiday\'->\'newYear\') as newYearHoliday");
      expectPass(function() {
        return parser.parseOneSelectField("holiday:newYear:$AS:newYearHoliday", true);
      }, "to_json(data->\'holiday\'->\'newYear\') as newYearHoliday");
      return done();
    });
    it('parseSelect should fail on bad arguments', function(done) {
      expectFail(function() {
        return parser.parseSelect(null);
      });
      return done();
    });
    it('parseSelect should pass on good arguments', function(done) {
      expectPass(function() {
        return parser.parseSelect("*");
      }, "*");
      expectPass(function() {
        return parser.parseSelect("price sku");
      }, "to_json(data->\'price\') as price,to_json(data->\'sku\') as sku");
      expectPass(function() {
        return parser.parseSelect("price:$as:unitPrice sku");
      }, "to_json(data->\'price\') as unitPrice,to_json(data->\'sku\') as sku");
      expectPass(function() {
        return parser.parseSelect("price:$as:unitPrice sku:$0:$AS:sku0");
      }, "to_json(data->\'price\') as unitPrice,to_json(data->\'sku\'->0) as sku0");
      return done();
    });
    it('parseOrderBy test', function(done) {
      expectFail(function() {
        return parser.parseOrderBy(null, true);
      });
      expectFail(function() {
        return parser.parseOrderBy("price sku");
      });
      expectFail(function() {
        return parser.parseOrderBy("*");
      });
      expectFail(function() {
        return parser.parseOrderBy("sdf*jfs");
      });
      expectPass(function() {
        return parser.parseOrderBy("price");
      }, "data->\'price\' ASC");
      expectPass(function() {
        return parser.parseOrderBy(" price:$0");
      }, "data->\'price\'->0 ASC");
      expectPass(function() {
        return parser.parseOrderBy(" price:sku:$3");
      }, "data->\'price\'->\'sku\'->3 ASC");
      expectPass(function() {
        return parser.parseOrderBy(" price DESC");
      }, "data->\'price\' DESC");
      return done();
    });
    it('parseLimit test', function(done) {
      expectFail(function() {
        return parser.parseLimit(null, true);
      });
      expectFail(function() {
        return parser.parseLimit("12o");
      });
      expectPass(function() {
        return parser.parseLimit(12);
      }, 12);
      expectPass(function() {
        return parser.parseLimit("12");
      }, 12);
      expectPass(function() {
        return parser.parseLimit(" 12 ");
      }, 12);
      return done();
    });
    it('parseOffset test', function(done) {
      expectFail(function() {
        return parser.parseOffset(null, true);
      });
      expectFail(function() {
        return parser.parseOffset("12o");
      });
      expectPass(function() {
        return parser.parseOffset(12);
      }, 12);
      expectPass(function() {
        return parser.parseOffset("12");
      }, 12);
      expectPass(function() {
        return parser.parseOffset(" 12 ");
      }, 12);
      return done();
    });
    it('parseIndexField test', function(done) {
      expectFail(function() {
        return parser.parseIndexField("", false);
      });
      expectPass(function() {
        return parser.parseIndexField("a", false);
      }, "data->\'a\'");
      expectPass(function() {
        return parser.parseIndexField("a", true);
      }, "data->>\'a\'");
      expectPass(function() {
        return parser.parseIndexField("  a ", true);
      }, "data->>\'a\'");
      expectPass(function() {
        return parser.parseIndexField("a:b ", false);
      }, "data->\'a\'->\'b\'");
      expectPass(function() {
        return parser.parseIndexField(" a:b", true);
      }, "data->\'a\'->>\'b\'");
      return done();
    });
    return it('parseTSIndex test', function(done) {
      expectFail(function() {
        return parser.parseTSIndex("", "english");
      });
      expectFail(function() {
        return parser.parseTSIndex("a:b", "simple");
      });
      expectFail(function() {
        return parser.parseTSIndex("a", "mars");
      });
      expectPass(function() {
        return parser.parseTSIndex("a", "english");
      }, "data->>\'a\'");
      expectPass(function() {
        return parser.parseTSIndex(" a ", "english");
      }, "data->>\'a\'");
      return done();
    });
  });

}).call(this);
