// Generated by CoffeeScript 1.7.1
(function() {
  var Promise, expect, pj, postgreConfig, uuid, _;

  expect = require('chai').expect;

  _ = require('lodash');

  uuid = require('node-uuid');

  Promise = require('bluebird');

  pj = require('../lib/pgJSON');

  Promise.promisifyAll(pj);

  postgreConfig = {
    hostname: 'localhost',
    port: 5432,
    username: 'huizhou',
    password: '123456',
    db: 'autotest'
  };

  describe('postgres json test', function() {
    var collection;
    pj.setDbUrl2(postgreConfig);
    collection = "unittest";
    before(function() {
      return pj.addCollectionAsync(collection);
    });
    afterEach(function() {
      return pj.clearCollectionAsync(collection);
    });
    after(function() {
      return pj.removeCollectionAsync(collection);
    });
    it('Basic document operation: add collection, insert, query by id, remove, remove collection should pass', function(done) {
      var collectionName, id, order;
      collectionName = "order";
      order = {
        quantity: 100,
        shipped: false,
        sku: "0109",
        contact: ["foo", "bar"],
        address: {
          number: '8'
        }
      };
      id = uuid.v4();
      return pj.addCollectionAsync(collectionName).then(function() {
        return pj.isCollectionExistAsync(collectionName);
      }).then(function(exist) {
        return expect(exist).to.equal(true);
      }).then(function() {
        return pj.insertAsync(id, collectionName, order);
      }).then(function(insertedDoc) {
        expect(_.isEqual(insertedDoc, order)).to.equal(true);
        return pj.queryByIdAsync(id, collectionName, null);
      }).then(function(doc) {
        return expect(_.isEqual(doc, order)).to.equal(true);
      }).then(function() {
        return pj.findOneAndUpdateAsync(id, collectionName, {
          sku: '1200',
          shipped: true,
          address: {
            street: 'victo',
            code: '1234'
          }
        });
      }).then(function(updatedDoc) {
        expect(updatedDoc.sku).to.equal('1200');
        expect(updatedDoc.shipped).to.equal(true);
        expect(updatedDoc.address.street).to.equal('victo');
        expect(updatedDoc.address.code).to.equal('1234');
        return expect(updatedDoc.address.number).to.equal('8');
      }).then(function() {
        return pj.removeByIdAsync(id, collectionName);
      }).then(function(removedCount) {
        return expect(removedCount).to.equal(1);
      }).then(function() {
        return pj.queryByIdAsync(id, collectionName, null);
      }).then(function(doc) {
        return expect(doc).to.equal(null);
      }).then(function() {
        return pj.removeCollectionAsync(collectionName);
      }).then(function() {
        return pj.isCollectionExistAsync(collectionName);
      }).then(function(exist) {
        expect(exist).to.equal(false);
        return done();
      })["catch"](function(error) {
        return done(error);
      });
    });
    it('findOneAndUpdateArray test should pass', function(done) {
      var collectionName, id, profile;
      collectionName = "contact";
      profile = {
        id: "123"
      };
      id = uuid.v4();
      return pj.addCollectionAsync(collectionName).then(function() {
        return pj.isCollectionExistAsync(collectionName);
      }).then(function(exist) {
        return expect(exist).to.equal(true);
      }).then(function() {
        return pj.insertAsync(id, collectionName, profile);
      }).then(function(inserted) {
        return pj.queryByIdAsync(id, collectionName, null);
      }).then(function(doc) {
        expect(_.isEqual(doc, profile)).to.equal(true);
        return pj.findOneAndUpdateArrayAsync(id, collectionName, {
          '$push': {
            'contacts': {
              id: 0,
              name: 'hui'
            }
          }
        });
      }).then(function(doc) {
        expect(doc.contacts.length).to.equal(1);
        return expect(_.some(doc.contacts, {
          id: 0,
          name: 'hui'
        })).to.equal(true);
      }).then(function() {
        return pj.findOneAndUpdateArrayAsync(id, collectionName, {
          '$push': {
            'contacts': {
              id: 2,
              name: 'zhou'
            }
          }
        });
      }).then(function(doc) {
        expect(doc.contacts.length).to.equal(2);
        expect(_.some(doc.contacts, {
          id: 2,
          name: 'zhou'
        })).to.equal(true);
        return pj.findOneAndUpdateArrayAsync(id, collectionName, {
          '$pull': {
            'contacts': {
              id: 0
            }
          }
        });
      }).then(function(doc) {
        expect(doc.contacts.length).to.equal(1);
        expect(_.some(doc.contacts, {
          id: 2,
          name: 'zhou'
        })).to.equal(true);
        return pj.findOneAndUpdateArrayAsync(id, collectionName, {
          '$set': {
            'contacts': {
              where: {
                id: 2
              },
              update: {
                name: 'dennis'
              }
            }
          }
        });
      }).then(function(doc) {
        expect(doc.contacts.length).to.equal(1);
        expect(_.some(doc.contacts, {
          id: 2,
          name: 'dennis'
        })).to.equal(true);
        return pj.findOneAndUpdateArrayAsync(id, collectionName, {
          '$push': {
            'contacts': [
              {
                id: 3,
                name: 'zhou'
              }, {
                id: 5,
                name: 'good'
              }
            ]
          }
        });
      }).then(function(doc) {
        return pj.findOneAndUpdateArrayAsync(id, collectionName, {
          '$pull': {
            'contacts': {
              id: 2
            }
          }
        });
      }).then(function(doc) {
        return expect(doc.contacts.length).to.equal(2);
      }).then(function() {
        return pj.removeByIdAsync(id, collectionName);
      }).then(function(removedCount) {
        return expect(removedCount).to.equal(1);
      }).then(function() {
        return pj.queryByIdAsync(id, collectionName, null);
      }).then(function(doc) {
        return expect(doc).to.equal(null);
      }).then(function() {
        return pj.removeCollectionAsync(collectionName);
      }).then(function() {
        return pj.isCollectionExistAsync(collectionName);
      }).then(function(exist) {
        expect(exist).to.equal(false);
        return done();
      })["catch"](function(error) {
        return done(error);
      });
    });
    it('Operation on non exist collection will fail', function(done) {
      var id;
      id = uuid.v4();
      return pj.addCollectionAsync("exist_collection").then(function() {
        return pj.insertAsync(id, "non_exist_collection", {
          quantity: 100
        })["catch"](function(error) {
          return expect(error).not.to.equal(null);
        });
      }).then(function() {
        return pj.insertAsync(id, "non_exist_collection", {
          quantity: 100
        })["catch"](function(error) {
          return expect(error).not.to.equal(null);
        });
      }).then(function() {
        return pj.upsertAsync(id, "non_exist_collection", {
          quantity: 100
        })["catch"](function(error) {
          return expect(error).not.to.equal(null);
        });
      }).then(function() {
        return pj.queryByIdAsync(id, "non_exist_collection", null)["catch"](function(error) {
          return expect(error).not.to.equal(null);
        });
      }).then(function(result) {
        return pj.removeCollectionAsync("exist_collection");
      }).then(function() {
        return done();
      });
    });
    it('update can only be applied to existing document', function(done) {
      var id, order;
      id = uuid.v4();
      order = {
        quantity: 100,
        shipped: false,
        sku: "0109",
        contact: ["foo", "bar"]
      };
      return pj.updateAsync(id, collection, order).then(function(updated) {
        return expect(updated).to.equal(0);
      }).then(function() {
        return pj.insertAsync(id, collection, {
          quantity: 100
        });
      }).then(function(inserted) {
        return pj.updateAsync(id, collection, order);
      }).then(function(updated) {
        expect(updated).to.equal(1);
        return pj.queryByIdAsync(id, collection, null);
      }).then(function(doc) {
        expect(_.isEqual(doc, order)).to.equal(true);
        return done();
      })["catch"](function(error) {
        return done(error);
      });
    });
    it('upsert can be applied to non existing document', function(done) {
      var id, order;
      id = uuid.v4();
      order = {
        quantity: 100,
        shipped: false,
        sku: "0109",
        contact: ["foo", "bar"]
      };
      return pj.upsertAsync(id, collection, order).then(function(upserted) {
        expect(upserted).to.equal(1);
        return pj.queryByIdAsync(id, collection, null);
      }).then(function(doc) {
        expect(_.isEqual(doc, order)).to.equal(true);
        return done();
      })["catch"](function(error) {
        return done(error);
      });
    });
    it('queryById selecting optional keys should pass', function(done) {
      var id, order;
      id = uuid.v4();
      order = {
        quantity: 100,
        shipped: false,
        sku: "0109",
        contact: ["foo", "bar", true, 90.5],
        address: {
          street: "foo street",
          postcode: 6035
        }
      };
      return pj.upsertAsync(id, collection, order).then(function(upserted) {
        expect(upserted).to.equal(1);
        return pj.queryByIdAsync(id, collection, "quantity sku");
      }).then(function(doc) {
        var supposeToBe;
        supposeToBe = {
          quantity: 100,
          sku: '0109'
        };
        expect(doc.quantity).to.equal(100);
        expect(doc.sku).to.equal('0109');
        expect(JSON.stringify(doc)).to.equal(JSON.stringify(supposeToBe));
        return pj.queryByIdAsync(id, collection, "quantity sku contact:$0:$as:contact0");
      }).then(function(doc) {
        var supposeToBe;
        supposeToBe = {
          quantity: 100,
          sku: '0109',
          contact0: 'foo'
        };
        expect(doc.quantity).to.equal(100);
        expect(doc.sku).to.equal('0109');
        expect(doc.contact0).to.equal('foo');
        expect(JSON.stringify(doc)).to.equal(JSON.stringify(supposeToBe));
        return pj.queryByIdAsync(id, collection, "quantity sku contact:$0:$as:contact0 contact:$3:$as:contact3 address:postcode:$as:postcode");
      }).then(function(doc) {
        var supposeToBe;
        supposeToBe = {
          quantity: 100,
          sku: '0109',
          contact0: 'foo',
          contact3: 90.5,
          postcode: 6035
        };
        expect(doc.quantity).to.equal(100);
        expect(doc.sku).to.equal('0109');
        expect(doc.contact3).to.equal(90.5);
        expect(doc.postcode).to.equal(6035);
        expect(JSON.stringify(doc)).to.equal(JSON.stringify(supposeToBe));
        return done();
      })["catch"](function(error) {
        return done(error);
      });
    });
    it('Query should pass on $eq / $ne test', function(done) {
      var product_0, product_1;
      product_0 = {
        _id: uuid.v4(),
        index: 0,
        active: true,
        info: {
          title: "foo",
          color: ["red", "blue"]
        },
        discount: [
          {
            quantity: 3,
            off: 0.2
          }, {
            quantity: 10,
            off: 0.3
          }
        ]
      };
      product_1 = {
        _id: uuid.v4(),
        index: 1,
        active: false,
        info: {
          title: "bar",
          color: ["green", "black"]
        },
        discount: [
          {
            quantity: 5,
            off: 0.1
          }, {
            quantity: 20,
            off: 0.5
          }
        ]
      };
      return pj.upsertAsync(product_0._id, collection, product_0).then(function() {
        return pj.upsertAsync(product_1._id, collection, product_1);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            index: {
              $eq: 1
            }
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        expect(_.isEqual(doc[0], product_1)).to.equal(true);
        return pj.queryAsync(collection, {
          where: {
            index: 1
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        return expect(_.isEqual(doc[0], product_1)).to.equal(true);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            active: {
              $EQ: true
            }
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        expect(_.isEqual(doc[0], product_0)).to.equal(true);
        return pj.queryAsync(collection, {
          where: {
            active: true
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        return expect(_.isEqual(doc[0], product_0)).to.equal(true);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            info: {
              title: {
                $eq: "foo"
              }
            }
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        return expect(_.isEqual(doc[0], product_0)).to.equal(true);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            info: {
              color: ["green", "black"]
            }
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        expect(_.isEqual(doc[0], product_1)).to.equal(true);
        return pj.queryAsync(collection, {
          where: {
            info: {
              color: {
                $eq: ["green", "black"]
              }
            }
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        return expect(_.isEqual(doc[0], product_1)).to.equal(true);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            discount: {
              $0: {
                off: {
                  $eq: 0.2
                }
              }
            }
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        expect(_.isEqual(doc[0], product_0)).to.equal(true);
        return pj.queryAsync(collection, {
          where: {
            discount: {
              $0: {
                off: 0.2
              }
            }
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        return expect(_.isEqual(doc[0], product_0)).to.equal(true);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            _id: {
              $eq: product_0._id
            }
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        return expect(_.isEqual(doc[0], product_0)).to.equal(true);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            index: {
              $ne: 1
            }
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        expect(_.isEqual(doc[0], product_0)).to.equal(true);
        return pj.queryAsync(collection, {
          where: {
            active: {
              $NE: true
            }
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        expect(_.isEqual(doc[0], product_1)).to.equal(true);
        return pj.queryAsync(collection, {
          where: {
            info: {
              title: {
                $ne: "foo"
              }
            }
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        expect(_.isEqual(doc[0], product_1)).to.equal(true);
        return pj.queryAsync(collection, {
          where: {
            info: {
              color: {
                $ne: ["green", "black"]
              }
            }
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        return expect(_.isEqual(doc[0], product_0)).to.equal(true);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            discount: {
              $0: {
                off: {
                  $ne: 0.2
                }
              }
            }
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        return expect(_.isEqual(doc[0], product_1)).to.equal(true);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            _id: {
              $ne: product_0._id
            }
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        expect(_.isEqual(doc[0], product_1)).to.equal(true);
        return done();
      })["catch"](function(error) {
        return done(error);
      });
    });
    it('findOneAndPopulateAsync test should be good', function(done) {
      var collectionName, ids, p0, p1, p2, p3;
      ids = [uuid.v4(), uuid.v4(), uuid.v4(), uuid.v4()];
      collectionName = "profile";
      p0 = {
        id: ids[0],
        name: '0',
        email: '0@0.com',
        contacts: [
          {
            _profile: ids[1]
          }, {
            _profile: ids[2]
          }
        ]
      };
      p1 = {
        id: ids[1],
        name: '1',
        email: '1@1.com',
        contacts: [
          {
            _profile: ids[0]
          }, {
            _profile: ids[2]
          }
        ]
      };
      p2 = {
        id: ids[2],
        name: '2',
        email: '2@2.com',
        contacts: [
          {
            _profile: ids[0]
          }, {
            _profile: ids[1]
          }
        ]
      };
      p3 = {
        id: ids[3],
        name: '3',
        email: '3@3.com',
        contacts: {
          _profile: ids[0]
        }
      };
      return pj.addCollectionAsync(collectionName).then(function() {
        return pj.insertAsync(p0.id, collectionName, p0);
      }).then(function() {
        return pj.insertAsync(p1.id, collectionName, p1);
      }).then(function() {
        return pj.insertAsync(p2.id, collectionName, p2);
      }).then(function() {
        return pj.insertAsync(p3.id, collectionName, p3);
      }).then(function() {
        return pj.findOneAndPopulateAsync(p0.id, collectionName, {
          key: 'contacts.$._profile',
          from: collectionName,
          select: 'name email'
        });
      }).then(function(result) {
        expect(result.id).to.equal(p0.id);
        expect(_.isEqual(result.contacts[0]._profile, {
          name: '1',
          email: '1@1.com'
        })).to.equal(true);
        expect(_.isEqual(result.contacts[1]._profile, {
          name: '2',
          email: '2@2.com'
        })).to.equal(true);
        return pj.findOneAndPopulateAsync(p1.id, collectionName, {
          key: 'contacts.$._profile',
          from: collectionName,
          select: 'name email'
        });
      }).then(function(result) {
        expect(result.id).to.equal(p1.id);
        expect(_.isEqual(result.contacts[0]._profile, {
          name: '0',
          email: '0@0.com'
        })).to.equal(true);
        expect(_.isEqual(result.contacts[1]._profile, {
          name: '2',
          email: '2@2.com'
        })).to.equal(true);
        return pj.findOneAndPopulateAsync(p2.id, collectionName, {
          key: 'contacts.$._profile',
          from: collectionName,
          select: 'name email'
        });
      }).then(function(result) {
        expect(result.id).to.equal(p2.id);
        expect(_.isEqual(result.contacts[0]._profile, {
          name: '0',
          email: '0@0.com'
        })).to.equal(true);
        expect(_.isEqual(result.contacts[1]._profile, {
          name: '1',
          email: '1@1.com'
        })).to.equal(true);
        return pj.findOneAndPopulateAsync(p3.id, collectionName, {
          key: 'contacts._profile',
          from: collectionName,
          select: 'name email'
        });
      }).then(function(result) {
        expect(result.id).to.equal(p3.id);
        return expect(_.isEqual(result.contacts._profile, {
          name: '0',
          email: '0@0.com'
        })).to.equal(true);
      }).then(function() {
        return pj.removeCollectionAsync(collectionName);
      }).then(function() {
        return done();
      })["catch"](function(error) {
        return done(error);
      });
    });
    it('Query should pass on $lt / le test', function(done) {
      var product_0, product_1;
      product_0 = {
        _id: uuid.v4(),
        index: 0,
        price: 20,
        discount: [
          {
            quantity: 5,
            off: 0.1
          }, {
            quantity: 10,
            off: 0.3
          }
        ]
      };
      product_1 = {
        _id: uuid.v4(),
        index: 1,
        price: 10,
        discount: [
          {
            quantity: 5,
            off: 0.2
          }, {
            quantity: 10,
            off: 0.3
          }
        ]
      };
      return pj.upsertAsync(product_0._id, collection, product_0).then(function() {
        return pj.upsertAsync(product_1._id, collection, product_1);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            price: {
              $lt: 10
            }
          }
        });
      }).then(function(doc) {
        expect(doc).to.eql([]);
        return pj.queryAsync(collection, {
          where: {
            price: {
              $le: 10
            }
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        return expect(_.isEqual(doc[0], product_1)).to.equal(true);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            price: {
              $gt: 20
            }
          }
        });
      }).then(function(doc) {
        expect(doc).to.eql([]);
        return pj.queryAsync(collection, {
          where: {
            price: {
              $ge: 20
            }
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        return expect(_.isEqual(doc[0], product_0)).to.equal(true);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            discount: {
              $0: {
                off: {
                  $LT: 0.15
                }
              }
            }
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        expect(_.isEqual(doc[0], product_0)).to.equal(true);
        return pj.queryAsync(collection, {
          where: {
            discount: {
              $0: {
                off: {
                  $GT: 0.15
                }
              }
            }
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        return expect(_.isEqual(doc[0], product_1)).to.equal(true);
      }).then(function() {
        return done();
      })["catch"](function(error) {
        return done(error);
      });
    });
    it('Query should pass on $all / $one test', function(done) {
      var product_0, product_1;
      product_0 = {
        _id: uuid.v4(),
        index: 0,
        email: '0@0.com',
        price: 20,
        quantity: 5,
        info: {
          title: "foo"
        }
      };
      product_1 = {
        _id: uuid.v4(),
        index: 0,
        email: '1@1.com',
        price: 20,
        info: {
          picture: "bar.jpg"
        },
        discount: [
          {
            quantity: 5,
            off: 0.1
          }, {
            quantity: 10,
            off: 0.3
          }
        ]
      };
      return pj.upsertAsync(product_0._id, collection, product_0).then(function() {
        return pj.upsertAsync(product_1._id, collection, product_1);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            $all: ["index", "price"]
          }
        });
      }).then(function(doc) {
        return expect(doc.length).to.equal(2);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            $ALL: ["index", "price", "quantity"]
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        return expect(_.isEqual(doc[0], product_0)).to.equal(true);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            $all: ["index", "price", "discount"]
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        return expect(_.isEqual(doc[0], product_1)).to.equal(true);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            $all: ["noexist"]
          }
        });
      }).then(function(doc) {
        return expect(doc).to.eql([]);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            $ANY: ["quantity", "discount"]
          }
        });
      }).then(function(doc) {
        return expect(doc.length).to.equal(2);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            info: {
              $any: ["title", "picture"]
            }
          }
        });
      }).then(function(doc) {
        return expect(doc.length).to.equal(2);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            email: {
              $any: ["0@0.com", "1@1.com"]
            }
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(2);
        return done();
      })["catch"](function(error) {
        return done(error);
      });
    });
    it('Query should pass on $inc test', function(done) {
      var order;
      order = {
        _id: uuid.v4(),
        quantity: 100,
        contact: [
          "foo", "bar", 100, false, {
            foo: "bar"
          }
        ]
      };
      return pj.insertAsync(order._id, collection, order).then(function() {
        return pj.queryAsync(collection, {
          where: {
            contact: {
              $inc: ["foo"]
            }
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        return expect(_.isEqual(doc[0], order)).to.equal(true);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            contact: {
              $INC: [100, false]
            }
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        return expect(_.isEqual(doc[0], order)).to.equal(true);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            contact: {
              $inc: [100, true]
            }
          }
        });
      }).then(function(doc) {
        return expect(doc).to.eql([]);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            contact: {
              $inc: 100
            }
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        return expect(_.isEqual(doc[0], order)).to.equal(true);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            contact: {
              $inc: false
            }
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        expect(_.isEqual(doc[0], order)).to.equal(true);
        return pj.queryAsync(collection, {
          where: {
            contact: {
              $inc: true
            }
          }
        });
      }).then(function(doc) {
        expect(doc).to.eql([]);
        return pj.queryAsync(collection, {
          where: {
            contact: {
              $inc: 90
            }
          }
        });
      }).then(function(doc) {
        expect(doc).to.eql([]);
        return pj.queryAsync(collection, {
          where: {
            contact: {
              $inc: {
                foo: "bar"
              }
            }
          }
        })["catch"](function(error) {
          return expect(error).not.to.eql([]);
        });
      }).then(function() {
        return done();
      })["catch"](function(error) {
        return done(error);
      });
    });
    it('$inc test 2: search array elements which contains provided key/value', function(done) {
      var order1, order2;
      order1 = {
        _id: uuid.v4(),
        quantity: 200,
        contact: [
          {
            id: 0,
            email: '0@0.com'
          }, {
            id: 1,
            email: '1@1.com'
          }
        ]
      };
      order2 = {
        _id: uuid.v4(),
        quantity: 12,
        contact: [
          {
            id: 0,
            email: '11@11.com'
          }, {
            id: 1,
            email: '22@22.com'
          }
        ]
      };
      return pj.insertAsync(order1._id, collection, order1).then(function() {
        return pj.insertAsync(order2._id, collection, order2);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            contact: {
              $inc: [
                {
                  email: "11@11.com"
                }
              ]
            }
          }
        });
      }).then(function(docs) {
        expect(docs.length).to.equal(1);
        expect(docs[0]._id).to.equal(order2._id);
        return pj.queryAsync(collection, {
          where: {
            contact: {
              $inc: [
                {
                  email: "1@1.com"
                }
              ]
            }
          }
        });
      }).then(function(docs) {
        expect(docs.length).to.equal(1);
        expect(docs[0]._id).to.equal(order1._id);
        return pj.queryAsync(collection, {
          where: {
            contact: {
              $inc: [
                {
                  email: "3@3.com"
                }
              ]
            }
          }
        });
      }).then(function(docs) {
        expect(docs).to.eql([]);
        return done();
      })["catch"](function(error) {
        return done(error);
      });
    });
    it('Query should pass on $not test', function(done) {
      var order_0, order_1, order_2;
      order_0 = {
        _id: uuid.v4(),
        quantity: 100
      };
      order_1 = {
        _id: uuid.v4(),
        quantity: 12
      };
      order_2 = {
        _id: uuid.v4(),
        quantity: 500
      };
      return pj.insertAsync(order_0._id, collection, order_0).then(function() {
        return pj.insertAsync(order_1._id, collection, order_1);
      }).then(function() {
        return pj.insertAsync(order_2._id, collection, order_2);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            $not: {
              quantity: {
                $GE: 90
              }
            }
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        return expect(_.isEqual(doc[0], order_1)).to.equal(true);
      }).then(function() {
        return done();
      })["catch"](function(error) {
        return done(error);
      });
    });
    it('Query should pass on $and test', function(done) {
      var order_0, order_1, order_2;
      order_0 = {
        _id: uuid.v4(),
        quantity: 100,
        price: 20
      };
      order_1 = {
        _id: uuid.v4(),
        quantity: 12,
        price: 70
      };
      order_2 = {
        _id: uuid.v4(),
        quantity: 500,
        price: 100
      };
      return pj.insertAsync(order_0._id, collection, order_0).then(function() {
        return pj.insertAsync(order_1._id, collection, order_1);
      }).then(function() {
        return pj.insertAsync(order_2._id, collection, order_2);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            $and: [
              {
                quantity: {
                  $ge: 90
                }
              }, {
                price: {
                  $LE: 50
                }
              }
            ]
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        return expect(_.isEqual(doc[0], order_0)).to.equal(true);
      }).then(function() {
        return done();
      })["catch"](function(error) {
        return done(error);
      });
    });
    it('Query should pass on $or test', function(done) {
      var order_0, order_1, order_2;
      order_0 = {
        _id: uuid.v4(),
        quantity: 100,
        price: 20
      };
      order_1 = {
        _id: uuid.v4(),
        quantity: 12,
        price: 70
      };
      order_2 = {
        _id: uuid.v4(),
        quantity: 500,
        price: 100
      };
      return pj.insertAsync(order_0._id, collection, order_0).then(function() {
        return pj.insertAsync(order_1._id, collection, order_1);
      }).then(function() {
        return pj.insertAsync(order_2._id, collection, order_2);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            $or: [
              {
                quantity: {
                  $GE: 400
                }
              }, {
                quantity: {
                  $le: 10
                }
              }
            ]
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        return expect(_.isEqual(doc[0], order_2)).to.equal(true);
      }).then(function() {
        return done();
      })["catch"](function(error) {
        return done(error);
      });
    });
    it('Query should pass on composite $or / $and test', function(done) {
      var ids, order_0, order_1, order_2, order_3;
      ids = [uuid.v4(), uuid.v4(), uuid.v4(), uuid.v4()];
      order_0 = {
        _id: ids[0],
        to: {
          type: 'system'
        },
        price: 100
      };
      order_1 = {
        _id: ids[1],
        to: {
          type: 'user',
          _id: ids[2]
        },
        price: 101
      };
      order_2 = {
        _id: ids[2],
        to: {
          type: 'user',
          _id: ids[3]
        },
        price: 102
      };
      order_3 = {
        _id: ids[3],
        to: {
          type: 'group',
          _id: ids[0]
        },
        price: 103
      };
      return pj.insertAsync(order_0._id, collection, order_0).then(function() {
        return pj.insertAsync(order_1._id, collection, order_1);
      }).then(function() {
        return pj.insertAsync(order_2._id, collection, order_2);
      }).then(function() {
        return pj.insertAsync(order_3._id, collection, order_3);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            $and: [
              {
                $or: [
                  {
                    to: {
                      type: 'system'
                    }
                  }, {
                    $and: [
                      {
                        to: {
                          type: 'user'
                        },
                        to: {
                          _id: ids[2]
                        }
                      }
                    ]
                  }
                ]
              }, {
                price: {
                  $le: 103
                }
              }
            ]
          }
        });
      }).then(function(doc) {
        return console.log(doc);
      }).then(function() {
        return done();
      })["catch"](function(error) {
        return done(error);
      });
    });
    it('test $any 2 should pass', function(done) {
      var order_0, order_1, order_2, order_3;
      order_0 = {
        _id: uuid.v4(),
        price: 1
      };
      order_1 = {
        _id: uuid.v4(),
        price: 2
      };
      order_2 = {
        _id: uuid.v4(),
        price: 3
      };
      order_3 = {
        _id: uuid.v4(),
        price: 4
      };
      return pj.insertAsync(order_0._id, collection, order_0).then(function() {
        return pj.insertAsync(order_1._id, collection, order_1);
      }).then(function() {
        return pj.insertAsync(order_2._id, collection, order_2);
      }).then(function() {
        return pj.insertAsync(order_3._id, collection, order_3);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            _id: {
              $any: [order_0._id, order_1._id]
            }
          }
        });
      }).then(function(docs) {
        expect(docs.length).to.equal(2);
        return console.log(docs);
      }).then(function() {
        return done();
      })["catch"](function(error) {
        return done(error);
      });
    });
    it('test $any 3 should pass', function(done) {
      var ids, order_0, order_1, order_2, order_3;
      ids = [uuid.v4(), uuid.v4(), uuid.v4(), uuid.v4()];
      order_0 = {
        _id: ids[0],
        to: [ids[1], ids[2]]
      };
      order_1 = {
        _id: ids[1],
        to: [ids[2], ids[3]]
      };
      order_2 = {
        _id: ids[2],
        to: [ids[3], ids[0]]
      };
      order_3 = {
        _id: ids[3],
        to: [ids[0], ids[1]]
      };
      return pj.insertAsync(order_0._id, collection, order_0).then(function() {
        return pj.insertAsync(order_1._id, collection, order_1);
      }).then(function() {
        return pj.insertAsync(order_2._id, collection, order_2);
      }).then(function() {
        return pj.insertAsync(order_3._id, collection, order_3);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            to: {
              $inc: [ids[0]]
            }
          }
        });
      }).then(function(docs) {
        expect(docs.length).to.equal(2);
        return console.log(docs);
      }).then(function() {
        return done();
      })["catch"](function(error) {
        return done(error);
      });
    });
    it.skip('test join', function(done) {
      var company_0, user_0;
      company_0 = {
        _id: uuid.v4(),
        name: 'test'
      };
      user_0 = {
        _id: uuid.v4(),
        company: company_0._id,
        name: 'hui'
      };
      return pj.addCollectionAsync('user').then(function() {
        return pj.addCollectionAsync('company');
      }).then(function() {
        return pj.insertAsync(company_0._id, 'company', company_0);
      }).then(function() {
        return pj.insertAsync(user_0._id, 'user', user_0);
      }).then(function() {
        return pj.pgQueryAsync("select u._id, u.data->>'name' as userName, c.data->>'name' as companyName from node_pg_json.user as u left join node_pg_json.company as c on (u.data->>'company')::uuid = c._id;", null);
      }).then(function(result) {
        return console.log(result.rows);
      }).then(function() {
        return pj.removeCollectionAsync('user');
      }).then(function() {
        return pj.removeCollectionAsync('company');
      }).then(function() {
        return done();
      })["catch"](function(error) {
        return done(error);
      });
    });
    it.skip('test join 2', function(done) {
      var company_0, user_0, user_1;
      user_0 = {
        _id: uuid.v4(),
        name: 'hui'
      };
      user_1 = {
        _id: uuid.v4(),
        name: 'ron'
      };
      company_0 = {
        _id: uuid.v4(),
        name: 'test',
        users: [user_0._id, user_1._id]
      };
      return pj.addCollectionAsync('user').then(function() {
        return pj.addCollectionAsync('company');
      }).then(function() {
        return pj.insertAsync(company_0._id, 'company', company_0);
      }).then(function() {
        return pj.insertAsync(user_0._id, 'user', user_0);
      }).then(function() {
        return pj.insertAsync(user_1._id, 'user', user_1);
      }).then(function() {
        return pj.pgQueryAsync("select c._id, c.data, json_agg(u.data) as users from node_pg_json.company as c left join lateral jsonb_array_elements_text(c.data->'users') as cu(userId) on true left join node_pg_json.user as u on u._id = (cu.userId)::uuid group by c._id, c.data;", null);
      }).then(function(result) {
        return console.log(JSON.stringify(result.rows));
      }).then(function() {
        return pj.removeCollectionAsync('user');
      }).then(function() {
        return pj.removeCollectionAsync('company');
      }).then(function() {
        return done();
      })["catch"](function(error) {
        return done(error);
      });
    });
    return it.skip('test join 3', function(done) {
      var company_0, user_0, user_1;
      user_0 = {
        _id: uuid.v4(),
        name: 'hui'
      };
      user_1 = {
        _id: uuid.v4(),
        name: 'ron'
      };
      company_0 = {
        _id: uuid.v4(),
        name: 'my company',
        group: {
          users: [
            {
              id: user_0._id
            }, {
              id: user_1._id
            }
          ]
        }
      };
      return pj.addCollectionAsync('user').then(function() {
        return pj.addCollectionAsync('company');
      }).then(function() {
        return pj.insertAsync(company_0._id, 'company', company_0);
      }).then(function() {
        return pj.insertAsync(user_0._id, 'user', user_0);
      }).then(function() {
        return pj.insertAsync(user_1._id, 'user', user_1);
      }).then(function() {
        return pj.pgQueryAsync("select c._id, c.data, json_agg(u.data) as users from node_pg_json.company as c left join lateral jsonb_array_elements_text(c.data->'user') as cu(userId) on true left join node_pg_json.user as u on u._id = (cu.userId)::uuid group by c._id, c.data;", null);
      }).then(function(result) {
        return console.log(JSON.stringify(result.rows));
      }).then(function() {
        return pj.removeCollectionAsync('user');
      }).then(function() {
        return pj.removeCollectionAsync('company');
      }).then(function() {
        return done();
      })["catch"](function(error) {
        return done(error);
      });
    });
  });

}).call(this);
