// Generated by CoffeeScript 1.7.1
(function() {
  var Promise, expect, pj, postgreConfig, uuid, _;

  expect = require('chai').expect;

  _ = require('lodash');

  uuid = require('node-uuid');

  Promise = require('bluebird');

  pj = require('../lib/pgJSON');

  Promise.promisifyAll(pj);

  postgreConfig = {
    hostname: 'localhost',
    port: 5432,
    username: 'huizhou',
    password: '123456',
    db: 'autotest'
  };

  describe('postgres json text search test', function() {
    var collection;
    pj.setDbUrl2(postgreConfig);
    collection = "unittest";
    before(function() {
      return pj.addCollectionAsync(collection);
    });
    beforeEach(function() {
      return pj.clearCollectionAsync(collection);
    });
    after(function() {
      return pj.removeCollectionAsync(collection);
    });
    it.skip("text search test", function(done) {
      var doc_0, doc_1, doc_2;
      doc_0 = {
        _id: uuid.v4(),
        index: 0,
        info: {
          title: "Compile a .coffee script"
        },
        content: "Compile a .coffee script into a .js JavaScript file of the same name."
      };
      doc_1 = {
        _id: uuid.v4(),
        index: 1,
        info: {
          title: "Generate source maps alongside"
        },
        content: "Generate source maps alongside the compiled JavaScript files. Adds sourceMappingURL directives to the JavaScript as well."
      };
      doc_2 = {
        _id: uuid.v4(),
        index: 2,
        info: {
          title: "Launch an interactive CoffeeScript"
        },
        content: "Launch an interactive CoffeeScript session to try short snippets. Identical to calling  with no arguments."
      };
      return pj.insertAsync(doc_0._id, collection, doc_0).then(function() {
        return pj.insertAsync(doc_1._id, collection, doc_1);
      }).then(function() {
        return pj.insertAsync(doc_2._id, collection, doc_2);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            content: {
              $text: {
                english: "file"
              }
            }
          }
        });
      }).then(function(doc) {
        return done();
      })["catch"](function(error) {
        return done(error);
      });
    });
    return it("pattern match test", function(done) {
      var doc_0, doc_1, doc_2;
      doc_0 = {
        _id: uuid.v4(),
        index: 0,
        info: {
          title: "foobar"
        }
      };
      doc_1 = {
        _id: uuid.v4(),
        index: 1,
        info: {
          title: "FOOBAR"
        }
      };
      doc_2 = {
        _id: uuid.v4(),
        index: 2,
        info: {
          title: "foobarer"
        }
      };
      return pj.insertAsync(doc_0._id, collection, doc_0).then(function() {
        return pj.insertAsync(doc_1._id, collection, doc_1);
      }).then(function() {
        return pj.insertAsync(doc_2._id, collection, doc_2);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            info: {
              title: {
                $like: "%er%"
              }
            }
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        return expect(_.isEqual(doc[0], doc_2)).to.equal(true);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            info: {
              title: {
                $like: "%AR%"
              }
            }
          }
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        return expect(_.isEqual(doc[0], doc_1)).to.equal(true);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            info: {
              title: {
                $ilike: "%AR%"
              }
            }
          },
          orderBy: "index asc"
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(3);
        expect(_.isEqual(doc[0], doc_0)).to.equal(true);
        expect(_.isEqual(doc[1], doc_1)).to.equal(true);
        return expect(_.isEqual(doc[2], doc_2)).to.equal(true);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            info: {
              title: {
                $ilike: "%AR%"
              }
            }
          },
          orderBy: "index desc"
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(3);
        expect(_.isEqual(doc[0], doc_2)).to.equal(true);
        expect(_.isEqual(doc[1], doc_1)).to.equal(true);
        return expect(_.isEqual(doc[2], doc_0)).to.equal(true);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            info: {
              title: {
                $ilike: "%AR%"
              }
            }
          },
          orderBy: "index desc",
          offset: 0,
          limit: 1
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        return expect(_.isEqual(doc[0], doc_2)).to.equal(true);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            info: {
              title: {
                $ilike: "%AR%"
              }
            }
          },
          orderBy: "index desc",
          offset: 1,
          limit: 1
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(1);
        return expect(_.isEqual(doc[0], doc_1)).to.equal(true);
      }).then(function() {
        return pj.queryAsync(collection, {
          where: {
            info: {
              title: {
                $ilike: "%AR%"
              }
            }
          },
          orderBy: "index desc",
          offset: 1,
          limit: 2
        });
      }).then(function(doc) {
        expect(doc.length).to.equal(2);
        expect(_.isEqual(doc[0], doc_1)).to.equal(true);
        return expect(_.isEqual(doc[1], doc_0)).to.equal(true);
      }).then(function() {
        return pj.countByQueryAsync(collection, {
          where: {
            info: {
              title: {
                $like: "%bar%"
              }
            }
          }
        });
      }).then(function(count) {
        return expect(count).to.equal(2);
      }).then(function() {
        return pj.removeByQueryAsync(collection, {
          where: {
            info: {
              title: {
                $like: "%bar%"
              }
            }
          }
        });
      }).then(function(count) {
        expect(count).to.equal(2);
        return done();
      })["catch"](function(error) {
        return done(error);
      });
    });
  });

}).call(this);
